proc (2) = dgls_l(y,x,nfirst,nlast,nbreak,nlagm,nar,maxit);
/*
   This procedure estimates model with a constant via GLS.
    The model is augmented with leads and lags of delta x
    It calculates Beta, tstat, and two Wald test statistics

Input:
y = y data (a column vector)
x = x data (levels of ) (may contain multiple regressors)
nfirst = first obs of y and x to use in regression
nlast = last obs of y and x to use in regression
nlag = maximum number of lags of delta x to use in regression
       (notice that lags include values of x before nfirst)
nar = number of ar lags
maxit = maximum number of corc iterations
nbreak = break date -- = 0 to skip this option

Output
beta = cointegrating vector estimates
varb = variance of beta ]

*/

local *;

bic=ones(nlagm,1);

jj=1;
do until jj>nlagm;
ya=y[nfirst:nlast,.];
xa=ones(nlast-nfirst+1,1)~x[nfirst:nlast,.];

@ check for structural breaks in the CIV if desired @
if (nbreak>(nfirst+jj+2))*(nbreak<(nlast-jj-2));
 xa9=x[(nbreak+1):nlast,.];@xa9=xa9-ones(rows(xa9),1)*(meanc(xa9)');@
 xa9=xa9-ones(rows(xa9),1)*(x[nbreak,.]);
 xa9=zeros(rows(xa)-rows(xa9),cols(x))|xa9;
 xa=xa~xa9;
endif;

@ Add lags to X matrix @
@   setup for all variables being I(1): @
dxa9=zeros(1,cols(x))|(x[2:rows(x),.]-x[1:(rows(x)-1),.]);
      /*
@ --- imposition of second differences of p ----- @
dxa91=0|0|(x[3:rows(x),1]-2*x[2:(rows(x)-1),1]+x[1:(rows(x)-2),1]);
dxa92=0|(x[2:rows(x),2]-x[1:(rows(x)-1),2]);
dxa93=0|(x[2:rows(x),3]-x[1:(rows(x)-1),3]);
@ ----- use of r-dp rather than deltar for final column -------- @
@dxa93=0|(x[2:rows(x),3]-100*(x[2:rows(x),1]-x[1:(rows(x)-1),1]));@
dxa9=dxa91~dxa92~dxa93;
        */

i=jj; do while i >= -jj;
 xa9=dxa9[nfirst+i:nlast+i,.];
@ xa9=(x[nfirst+i:nlast+i,.]-x[nfirst+i-1:nlast+i-1,.]);@
 xa=xa~xa9;
i=i-1; endo;

beta=(xa'ya)/(xa'xa);  @ OLS estimate @

@ Carry Out Corc iterations @

it=1; do while it <= maxit;
e=ya-xa*beta;
xe=e[nar:rows(e)-1,1];
i=2; do while i <= nar;
 xe=xe~e[nar+1-i:rows(e)-i,1];
i=i+1; endo;
rho=(xe'e[nar+1:rows(e),1])/(xe'xe);
"it -- rho";; it~rho';
yar=ya[nar+1:rows(ya),.];
xar=xa[nar+1:rows(xa),.];

@ quasi difference data @
i=1; do while i <= nar;
 yar=yar-rho[i,1]*ya[nar+1-i:rows(ya)-i,.];
 xar=xar-rho[i,1]*xa[nar+1-i:rows(xa)-i,.];
i=i+1; endo;

@ Estimate Beta via GLS @
beta=(xar'yar)/(xar'xar);
if it .== maxit;
 dfcorr=rows(yar)-cols(xar)-nar;
rows(yar) cols(xar) nar dfcorr;
 s2=(yar'yar-beta'(xar'yar))/dfcorr;
 xxi=invpd(xar'xar);
endif;

it=it+1; endo;

varb=s2*xxi;
"T * ln(sighat) " dfcorr*ln(sqrt(s2));

if (nbreak>(nfirst+jj+2))*(nbreak<(nlast-jj-2));
 format /RDN 3,0;
 " ";"Wald test for change in cointegrating vector at n = " nbreak;
 "   Test of equality of cointegrating vector in two subsamples";
 "   chisq (df =" cols(x) " ) = ";;
 i1=2+cols(x);i2=2+2*cols(x);
 chi9=beta[i1:i2,.]'inv(varb[i1:i2,i1:i2])*beta[i1:i2,.];
 p9=cdfchic(chi9,cols(x));
 format /RDN 8,3; chi9 "  p-value = " p9;
endif;

bic[jj,1] = ln(det(s2))+
   ((1+cols(xar)+2*cols(xar)*jj)*(ln(rows(yar))/rows(yar)));
jj;;bic[jj,1];
jj=jj+1;
endo;

/* select nlag as min bic and then resestimate model */
nlag=minindc(bic);
" bic are ";bic;
"lags selected = ";nlag;

ya=y[nfirst:nlast,.];
xa=ones(nlast-nfirst+1,1)~x[nfirst:nlast,.];

@ check for structural breaks in the CIV if desired @
if (nbreak>(nfirst+nlag+2))*(nbreak<(nlast-nlag-2));
 xa9=x[(nbreak+1):nlast,.];@xa9=xa9-ones(rows(xa9),1)*(meanc(xa9)');@
 xa9=xa9-ones(rows(xa9),1)*(x[nbreak,.]);
 xa9=zeros(rows(xa)-rows(xa9),cols(x))|xa9;
 xa=xa~xa9;
endif;

@ Add lags to X matrix @
@   setup for all variables being I(1): @
dxa9=zeros(1,cols(x))|(x[2:rows(x),.]-x[1:(rows(x)-1),.]);
      /*
@ --- imposition of second differences of p ----- @
dxa91=0|0|(x[3:rows(x),1]-2*x[2:(rows(x)-1),1]+x[1:(rows(x)-2),1]);
dxa92=0|(x[2:rows(x),2]-x[1:(rows(x)-1),2]);
dxa93=0|(x[2:rows(x),3]-x[1:(rows(x)-1),3]);
@ ----- use of r-dp rather than deltar for final column -------- @
@dxa93=0|(x[2:rows(x),3]-100*(x[2:rows(x),1]-x[1:(rows(x)-1),1]));@
dxa9=dxa91~dxa92~dxa93;
        */

i=nlag; do while i >= -nlag;
 xa9=dxa9[nfirst+i:nlast+i,.];
@ xa9=(x[nfirst+i:nlast+i,.]-x[nfirst+i-1:nlast+i-1,.]);@
 xa=xa~xa9;
i=i-1; endo;

beta=(xa'ya)/(xa'xa);  @ OLS estimate @

@ Carry Out Corc iterations @

it=1; do while it <= maxit;
e=ya-xa*beta;
xe=e[nar:rows(e)-1,1];
i=2; do while i <= nar;
 xe=xe~e[nar+1-i:rows(e)-i,1];
i=i+1; endo;
rho=(xe'e[nar+1:rows(e),1])/(xe'xe);
"it -- rho";; it~rho';
yar=ya[nar+1:rows(ya),.];
xar=xa[nar+1:rows(xa),.];

@ quasi difference data @
i=1; do while i <= nar;
 yar=yar-rho[i,1]*ya[nar+1-i:rows(ya)-i,.];
 xar=xar-rho[i,1]*xa[nar+1-i:rows(xa)-i,.];
i=i+1; endo;

@ Estimate Beta via GLS @
beta=(xar'yar)/(xar'xar);
if it .== maxit;
 dfcorr=rows(yar)-cols(xar)-nar;
rows(yar) cols(xar) nar dfcorr;
 s2=(yar'yar-beta'(xar'yar))/dfcorr;
 xxi=invpd(xar'xar);
endif;

it=it+1; endo;

varb=s2*xxi;
"T * ln(sighat) " dfcorr*ln(sqrt(s2));

if (nbreak>(nfirst+nlag+2))*(nbreak<(nlast-nlag-2));
 format /RDN 3,0;
 " ";"Wald test for change in cointegrating vector at n = " nbreak;
 "   Test of equality of cointegrating vector in two subsamples";
 "   chisq (df =" cols(x) " ) = ";;
 i1=2+cols(x);i2=2+2*cols(x);
 chi9=beta[i1:i2,.]'inv(varb[i1:i2,i1:i2])*beta[i1:i2,.];
 p9=cdfchic(chi9,cols(x));
 format /RDN 8,3; chi9 "  p-value = " p9;
endif;




@ retp(beta[1:cols(x)+1,.],varb[1:cols(x)+1,1:cols(x)+1]); @
  retp(beta,varb);

endp;



@ debug @
{beta,varb}=dgls_l(y,x,7,85,0,5,5,50);
beta[1:5,.]~sqrt(diag(varb[1:5,.]));






