@ dgls_lf.prc      mww, mod. jhs 10/13/90
    returns value of conditional likelihood function for dgls estn
    modification of DGLS.PRC
@

proc (1) = dgls_lf(y,x,nfirst,nlast,nlag,nar,maxit,b);

/*
   This procedure estimates model with a constant via GLS.
    The model is augmented with leads and lags of delta x
    It calculates Beta, tstat, and two Wald test statistics

Input:
y = y data (a column vector)
x = x data (levels of ) (may contain multiple regressors)
nfirst = first obs of y and x to use in regression
nlast = last obs of y and x to use in regression
nlag = number of lags of delta x to use in regression
       (notice that lags include values of x before nfirst)
nar = number of ar lags
maxit = maximum number of corc iterations
b = CIV parameter vector for partial likelihood evaluation

Output
s2 = ss errors of epsilon hat
sumc(rho) = sum of AR coefficients in AR correction

*/

local *;

ya=y[nfirst:nlast,.]-x[nfirst:nlast,.]*b;
xa=ones(nlast-nfirst+1,1);

@ Add lags to X matrix @
i=nlag; do while i >= -nlag;
 xa=xa~(x[nfirst+i:nlast+i,.]-x[nfirst+i-1:nlast+i-1,.]);
i=i-1; endo;
beta=(xa'ya)/(xa'xa);  @ OLS estimate @

@ Carry Out Corc iterations @

it=1; do while it <= maxit;
e=ya-xa*beta;
xe=e[nar:rows(e)-1,1];
i=2; do while i <= nar;
 xe=xe~e[nar+1-i:rows(e)-i,1];
i=i+1; endo;
rho=(xe'e[nar+1:rows(e),1])/(xe'xe);
@"it -- rho";; it~rho';@
yar=ya[nar+1:rows(ya),.];
xar=xa[nar+1:rows(xa),.];

@ quasi difference data @
i=1; do while i <= nar;
 yar=yar-rho[i,1]*ya[nar+1-i:rows(ya)-i,.];
 xar=xar-rho[i,1]*xa[nar+1-i:rows(xa)-i,.];
i=i+1; endo;

@ Estimate Beta via GLS @
beta=(xar'yar)/(xar'xar);
if it .== maxit;
 sseps=yar'yar-beta'(xar'yar);
 s2=sseps/(rows(yar));
 xxi=invpd(xar'xar);
endif;

it=it+1; endo;

varb=s2*xxi;
lfn=-0.5*rows(yar)*ln(s2);

@ retp(beta[1:cols(x)+1,.],varb[1:cols(x)+1,1:cols(x)+1]); @
@  retp(beta,varb);@

retp(lfn|sumc(rho));
endp;

