proc (2) = dols_ar(y,x,nfirst,nlast,nbreak,nlag,nar);
/*
   This procedure estimates model with a constant via OLS.
    The model is augmented with leads and lags of delta x
    The covariance matrix is calculated robustly (serial cor).
    It calculates Beta, tstat, and two Wald test statistics

Input:
y = y data              Tx1
x = x data (levels of ) Txk
nfirst = first obs of y and x to use in regression
nlast = last obs of y and x to use in regression
nlag = number of lags of delta x to use in regression
       (notice that lags include values of x before nfirst)
nlcov = number of covariances to use in spectral estimate
kern = kernel used in spectral estimate
nbreak = break date -- = 0 to skip this option

Output:
beta == cointegrating vector estimator
varb == covariance matrix of beta

*/

local *;
ya=y[nfirst:nlast,.];
xa=ones(nlast-nfirst+1,1)~x[nfirst:nlast,.];

@ check for structural breaks in the CIV if desired @
if (nbreak>(nfirst+nlag+2))*(nbreak<(nlast-nlag-2));
 xa9=x[(nbreak+1):nlast,.];@xa9=xa9-ones(rows(xa9),1)*(meanc(xa9)');@
 xa9=xa9-ones(rows(xa9),1)*(x[nbreak,.]);
 xa9=zeros(rows(xa)-rows(xa9),cols(x))|xa9;
 xa=xa~xa9;
endif;

nxparm=cols(xa);

@ Add lags to X matrix @
@   setup for all variables being I(1): @
dxa9=zeros(1,cols(x))|(x[2:rows(x),.]-x[1:(rows(x)-1),.]);
      /*
@ --- imposition of second differences of p ----- @
dxa91=0|0|(x[3:rows(x),1]-2*x[2:(rows(x)-1),1]+x[1:(rows(x)-2),1]);
dxa92=0|(x[2:rows(x),2]-x[1:(rows(x)-1),2]);
dxa93=0|(x[2:rows(x),3]-x[1:(rows(x)-1),3]);
@ ----- use of r-dp rather than deltar for final column -------- @
@dxa93=0|(x[2:rows(x),3]-100*(x[2:rows(x),1]-x[1:(rows(x)-1),1]));@
dxa9=dxa91~dxa92~dxa93;
        */

i=nlag; do while i >= -nlag;
 xa9=dxa9[nfirst+i:nlast+i,.];
@ xa9=(x[nfirst+i:nlast+i,.]-x[nfirst+i-1:nlast+i-1,.]);@
 xa=xa~xa9;
i=i-1; endo;


xxi=invpd(xa'xa);
xy=xa'ya;
beta=xxi*xy;    @ OLS Estimate @

@!!!!!!!!!  Calculate autocor. robust standard errors !!!!!!!!!!@
e=ya-xa*beta;

  @ Calculate spectral density of e at frequency zero @
/*
s2=(e'e)*kern[1,1];
i=1; do while i <= nlcov;
 s2=s2+2*kern[i+1,1]*(e[1+i:rows(e),1]'e[1:rows(e)-i,1]);
i=i+1; endo;
s2=s2/rows(e);   @ Spectrum of Residuals at frequency zero @
*/
@ autoregressive spectral estimator @
ey=e[nar+1:rows(e),.]; ex=e[nar:(rows(e)-1),.];
il=2; do until il>nar;
 ex=ex~e[(nar-il+1):(rows(e)-il),.];
il=il+1; endo; @ex=ex~ones(rows(ex),1);@
phihat=inv(ex'*ex)*(ex'*ey); ee=ey-ex*phihat;
s2n=ee'*ee/(rows(ee)-cols(xa)-nar);
s2d=(1-sumc(phihat[1:(rows(phihat)),.]));s2=s2n/(s2d*s2d);
@rows(ex) cols(ex);@phihat;
@(rows(e)-cols(xa));sqrt(s2n);@
varb=s2*xxi;

if (nbreak>(nfirst+nlag+2))*(nbreak<(nlast-nlag-2));
 format /RDN 3,0;
 " ";"Wald test for change in cointegrating vector at n = " nbreak;
 "   Test of equality of cointegrating vector in two subsamples";
 "   chisq (df =" cols(x) " ) = ";;
 i1=2+cols(x);i2=2+2*cols(x);
 chi9=beta[i1:i2,.]'inv(varb[i1:i2,i1:i2])*beta[i1:i2,.];
 p9=cdfchic(chi9,cols(x));
 format /RDN 8,3; chi9 "  p-value = " p9;
endif;

retp(beta[1:nxparm,.],varb[1:nxparm,1:nxparm]);

endp;

/*
@ debug @
ib=0; do until ib>0;
  {b,vb}=dols_ar(y,x,5,88,ib,2,2);
  b~sqrt(diag(vb));
ib=ib+1; endo;
*/


