@ DOLS_LF.prc    computes value of ssr from DOLS regression
   modification of dols.prc  JHS 10/14/90
@
proc (1) = dols_lf(y,x,nfirst,nlast,nlag,nlcov,kern,b);
/*
   This procedure estimates model with a constant via OLS.
    The model is augmented with leads and lags of delta x
    The covariance matrix is calculated robustly (serial cor).
    It calculates Beta, tstat, and two Wald test statistics

Input:
y = y data              Tx1
x = x data (levels of ) Txk
nfirst = first obs of y and x to use in regression
nlast = last obs of y and x to use in regression
nlag = number of lags of delta x to use in regression
       (notice that lags include values of x before nfirst)
nlcov = number of covariances to use in spectral estimate
kern = kernel used in spectral estimate

Output:
beta == cointegrating vector estimator
varb == covariance matrix of beta

*/

local *;
ya=y[nfirst:nlast,.]-x[nfirst:nlast,.]*b;
xa=ones(rows(ya),1);

@ Add lags to X matrix @
i=nlag; do while i >= -nlag;
 xa=xa~(x[nfirst+i:nlast+i,.]-x[nfirst+i-1:nlast+i-1,.]);
i=i-1; endo;
xxi=invpd(xa'xa);
xy=xa'ya;
beta=xxi*xy;    @ OLS Estimate @

@!!!!!!!!!  Calculate autocor. robust standard errors !!!!!!!!!!@
e=ya-xa*beta;
s2=(e'e)/rows(e);
ssr=-rows(e)*ln(sqrt(s2));

 @ calculate tdf statistic for residuals, rhohat @
tdfrho=adf_reg(e,nlag+2,rows(e),nlag);

/*
  @ Calculate spectral density of e ar frequency zero @

s2=(e'e)*kern[1,1];
i=1; do while i <= nlcov;
 s2=s2+2*kern[i+1,1]*(e[1+i:rows(e),1]'e[1:rows(e)-i,1]);
i=i+1; endo;
s2=s2/rows(e);   @ Spectrum of Residuals at frequency zero @

varb=s2*xxi;
*/

retp(ssr|tdfrho);

endp;

/*
@ debug: @
dols_lf(y,x,nf,nl,nlag,nlcov,kern,1);
*/

