proc (2) = pbsr(y,x,nfirst,nlast,nlcov,kern);
/*
   This procedure estimates cointegrating vector for a model with a constant
    via Phillips' spectral regression procedure. (See Phillips (1988)
    "Spectral Regression fir Cointegrated Time Series"
     The estimator used is the zero frequency band spectral est.

Input:
y = y data              Tx1
x = x data (levels of ) Txk
xd = true drift in x series (used to constuct the
     true zero-mean errors in x (kx1)
nfirst = first obs of y and x to use in regression
nlast = last obs of y and x to use in regression
nlcov = number of covariances to use in spectral estimate
kern = kernel used in spectral estimate

Output:
beta = cointegrating vector estimator
varb = covariance matrix for beta

*/

local beta, xa, ya, xxi, xy, s2, varb;
local u, v, bbols, f22, fvv, f2s, e, xl, fvvi, i, uhat, vhat, varb1;
local vt;
ya=y[nfirst:nlast,.];
xa=ones(nlast-nfirst+1,1)~x[nfirst:nlast,.];
xl=ones(nlast-nfirst+1,1)~x[nfirst-1:nlast-1,.];
u=zeros(rows(ya),cols(x)+1);
v=zeros(rows(ya),cols(x)+1);
u[.,1]=ya;
u[.,2:cols(u)]=x[nfirst:nlast,.]-x[nfirst-1:nlast-1,.];
u[.,2:cols(u)]=u[.,2:cols(u)]
               -ones(rows(u),cols(u)-1).*(meanc(u[.,2:cols(u)]))';
bbols=ya/xl;
  vhat=(ya-xl*bbols)~(u[.,2:cols(u)]-meanc(u[.,2:cols(u)])');
@ uhat=(ya-xa*bbols)~(u[.,2:cols(u)]);
  vhat=(u[.,1]+u[.,2:cols(u)]*bbols[2:rows(bbols),.])
   ~u[.,2:cols(u)]; @
v=u;

@ Calculate necessary Spectral Density Matrices @
fvv=vhat'vhat;
f22=xl'xl;
f2s=xl'v;

i=1; do while i <= nlcov;
 fvv=fvv+kern[i+1,1]*(vhat[1+i:rows(v),.]'vhat[1:rows(v)-i,.]+
                      vhat[1:rows(v)-i,.]'vhat[1+i:rows(v),.]);
 f22=f22+kern[i+1,1]*(xl[1+i:rows(v),.]'xl[1:rows(v)-i,.]+
                      xl[1:rows(v)-i,.]'xl[1+i:rows(v),.]);
 f2s=f2s+kern[i+1,1]*(xl[1+i:rows(v),.]'v[1:rows(v)-i,.]+
                      xl[1:rows(v)-i,.]'v[1+i:rows(v),.]);
i=i+1; endo;
fvv=fvv/rows(v);
f22=f22/rows(v);
f2s=f2s/rows(v);

e=zeros(rows(fvv),1); e[1,1]=1;
fvvi=inv(fvv);
beta=inv(f22)*f2s*fvvi*e/(e'fvvi*e);
s2 = inv(e'fvvi*e);
varb1=(2*sumc(kern)-kern[1,1])*inv(f22)/rows(v);
varb=s2*varb1;

retp(beta, varb);
endp;
/*
{beta,varb}=pbsr(y,x,nf,nl,5,kern);
beta~sqrt(diag(varb));
*/

