proc (2) = phfm(y,x,nfirst,nlast,nlcov,kern);
/*
   This procedure estimates cointegrating vector for a model with a constant
    using the "Fully modified" OLS estimator described in 
    Phillips and Hansen "Statistical Inference in Instrumental
    Variable Regression with I(1) Processes" (rev. April 1989)

Input:
y = y data              Tx1
x = x data (levels of ) Txk
nfirst = first obs of y and x to use in regression
nlast = last obs of y and x to use in regression
nlcov = number of covariances to use in spectral estimate
kern = kernel used in spectral estimate


Output:

beta =  cointegrating coefficients
varb   = covariance matrix of b 

*/

local beta, xa, ya, tstat, varb;
local uhat, bbols, omega, dlta, omega22, omega12, dlta2b, j1b, i, y1hat;
local s2, varb1, vt;
ya=y[nfirst:nlast,.];
xa=ones(nlast-nfirst+1,1)~x[nfirst:nlast,.];
bbols=ya/xa;
uhat=(ya-xa*bbols)~(x[nfirst:nlast,.]-x[nfirst-1:nlast-1,.] -
               meanc(x[nfirst:nlast,.]-x[nfirst-1:nlast-1,.])');
@ Calculate necessary omega and delta @
omega=uhat'uhat;
dlta =uhat'uhat;

i=1; do while i <= nlcov;
 omega=omega+kern[i+1,1]*(uhat[1+i:rows(uhat),.]'uhat[1:rows(uhat)-i,.]+
                      uhat[1:rows(uhat)-i,.]'uhat[1+i:rows(uhat),.]);
 dlta=dlta+kern[i+1,1]*(uhat[1:rows(uhat)-i,.]'uhat[1+i:rows(uhat),.]);
i=i+1; endo;
omega=omega/(rows(uhat));
dlta=dlta/(rows(uhat));
omega22=omega[2:rows(omega),2:cols(omega)];
omega12=omega[1,2:cols(omega)];
j1b=eye(1)~(-omega12*(inv(omega22)));
dlta2b=dlta[2:rows(dlta),.];
@ Construct Estimator @

y1hat=ya-uhat[.,2:cols(uhat)] * (inv(omega22))*omega12';
beta=inv(xa'xa)*(xa'y1hat-rows(y1hat)*(0|dlta2b*j1b'));

s2=omega[1,1]-omega12*(inv(omega22))*omega12';
varb1=inv(xa'xa);
varb=s2*varb1;

retp(beta,varb);
endp;

